;;;; ftp-parse.scm
;;;; Kon Lovett, Feb '07

(use srfi-1 regex extras)

(eval-when (compile)
  (declare
    (usual-integrations)
    (no-procedure-checks-for-usual-bindings)
    (fixnum)
    (inline)
    (no-procedure-checks)
    (no-bound-checks)
    (export
      parse-ftp-address) ) )

;;;

(define (scan-list re str)
  (take-while! identity (cdr (or (string-match re str) '(())))) )

(define (%scan-set! re str setrs)
  (let ([num 0])
    (let loop ([lst (scan-list re str)] [setrs setrs])
      (if (null? setrs)
        num
        (if (null? lst)
          (loop '() (cdr setrs))
          (begin
            ((car setrs) (car lst))
            (set! num (add1 num))
            (loop (cdr lst) (cdr setrs)) ) ) ) ) ) )

(define-macro (scan-set! str re . vars)
  (let loop ([vars vars] [setrs '()])
    (if (null? vars)
      `(%scan-set! ,re ,str (list ,@(reverse! setrs)))
      (loop (cdr vars) (cons `(lambda (x) (set! ,(car vars) x)) setrs)) ) ) )

;;@noindent
;;Before RFC 2396, the @dfn{File Transfer Protocol} (FTP) served a
;;similar purpose.

;;@body
;;Returns a list of the decoded FTP @1; or #f if indecipherable.  FTP
;;@dfn{Uniform Resource Locator}, @dfn{ange-ftp}, and @dfn{getit}
;;formats are handled.  The returned list has four elements which are
;;strings or #f:
;;
;;@enumerate 0
;;@item
;;username
;;@item
;;password
;;@item
;;remote-site
;;@item
;;remote-directory
;;@end enumerate

(define parse-ftp-address
  (let (
      [re1 (regexp "[ \t\n]*ftp://([^ \t\n]+)?[ \t\n]*([^ \t\n]+)?" #f #f #t)]
      [re2 (regexp "([^/]+)?/([^@]+)?([^ \t\n]+)?" #f #f #t)]
      [re3 (regexp "([^@]+)?@([^@]+)?([^ \t\n]+)?" #f #f #t)]
      [re4 (regexp "([^:]+)?:([^@/]+)?([^ \t\n]+)?" #f #f #t)]
      [re5 (regexp "[ \t\n]*([^:]+)?([:]+)?([^@]+)?[ \t\n]*([^ \t\n]+)?" #f #f #t)]
      [re6 (regexp "/([^@/]+)?@([^@]+)?([^ \t\n]+)?" #f #f #t)]
      [re7 (regexp "([^@/]+)?@([^@]+)?([^ \t\n]+)?" #f #f #t)]
      [re8 (regexp "@([^@]+)?([^ \t\n]+)?" #f #f #t)]
      [re9 (regexp "[ \t\n]*([^@/]+)?[ \t\n]*([^ \t\n]+)?" #f #f #t)])
    (lambda (uri-str)
      (let ((length? (lambda (len lst) (and (eqv? len (length lst)) lst))))
        (cond
         ((not uri-str) #f)
         ((length? 1 (scan-list re1 uri-str))
          => (lambda (host)
               (let ((login #f) (path #f) (dross #f))
                 (scan-set! (car host) re2 login path dross)
                 (and login
                      (append (cond
                               ((length? 2 (scan-list re3 login))
                                => (lambda (userpass@hostport)
                                     (append
                                      (cond ((length? 2 (scan-list
                                                         re4
                                                         (car userpass@hostport))))
                                            (else (list (car userpass@hostport) #f)))
                                      (cdr userpass@hostport))))
                               (else (list "anonymous" #f login)))
                              (list path))))))
         (else
          (let ((user@site #f) (colon #f) (path #f) (dross #f))
            (case (scan-set! uri-str re5 user@site colon path dross)
              ((2 3)
               (let ((user #f) (site #f))
                 (cond ((or (eqv? 2 (scan-set! user@site re6
                                            user site dross))
                            (eqv? 2 (scan-set! user@site re7
                                            user site dross)))
                        (list user #f site path))
                       ((eqv? 1 (scan-set! user@site re8 site dross))
                        (list #f #f site path))
                       (else (list #f #f user@site path)))))
              (else
               (let ((site (scan-list re9 uri-str)))
                 (and (length? 1 site)
                      (list #f #f (car site) #f)))))))) ) ) ) )
